"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateCommand = exports.plainGeneratorKeysSchema = void 0;
const lisk_codec_1 = require("@liskhq/lisk-codec");
const lisk_cryptography_1 = require("@liskhq/lisk-cryptography");
const core_1 = require("@oclif/core");
const fs = require("fs-extra");
const path = require("path");
const flags_1 = require("../../../utils/flags");
const reader_1 = require("../../../utils/reader");
exports.plainGeneratorKeysSchema = {
    $id: '/commander/plainGeneratorKeys',
    type: 'object',
    properties: {
        generatorKey: {
            dataType: 'bytes',
            fieldNumber: 1,
        },
        generatorPrivateKey: {
            dataType: 'bytes',
            fieldNumber: 2,
        },
        blsKey: {
            dataType: 'bytes',
            fieldNumber: 3,
        },
        blsPrivateKey: {
            dataType: 'bytes',
            fieldNumber: 4,
        },
    },
};
class CreateCommand extends core_1.Command {
    async run() {
        const { flags: { output, passphrase: passphraseSource, 'no-encrypt': noEncrypt, password: passwordSource, count, offset, chainid, }, } = await this.parse(CreateCommand);
        if (output) {
            const { dir } = path.parse(output);
            fs.ensureDirSync(dir);
        }
        const passphrase = passphraseSource !== null && passphraseSource !== void 0 ? passphraseSource : (await (0, reader_1.getPassphraseFromPrompt)('passphrase', true));
        let password = '';
        if (!noEncrypt) {
            password = passwordSource !== null && passwordSource !== void 0 ? passwordSource : (await (0, reader_1.getPasswordFromPrompt)('password', true));
        }
        const keys = [];
        for (let i = 0; i < count; i += 1) {
            const accountKeyPath = `m/44'/134'/${offset + i}'`;
            const generatorKeyPath = `m/25519'/134'/${chainid}'/${offset + i}'`;
            const blsKeyPath = `m/12381/134/${chainid}/${offset + i}`;
            const accountPrivateKey = await lisk_cryptography_1.ed.getPrivateKeyFromPhraseAndPath(passphrase, accountKeyPath);
            const accountPublicKey = lisk_cryptography_1.ed.getPublicKeyFromPrivateKey(accountPrivateKey);
            const address = lisk_cryptography_1.address.getAddressFromPublicKey(accountPublicKey);
            const generatorPrivateKey = await lisk_cryptography_1.ed.getPrivateKeyFromPhraseAndPath(passphrase, generatorKeyPath);
            const generatorPublicKey = lisk_cryptography_1.ed.getPublicKeyFromPrivateKey(generatorPrivateKey);
            const blsPrivateKey = await lisk_cryptography_1.bls.getPrivateKeyFromPhraseAndPath(passphrase, blsKeyPath);
            const blsPublicKey = lisk_cryptography_1.bls.getPublicKeyFromPrivateKey(blsPrivateKey);
            let encryptedMessageObject = {};
            if (!noEncrypt) {
                const plainGeneratorKeyData = {
                    generatorKey: generatorPublicKey,
                    generatorPrivateKey,
                    blsKey: blsPublicKey,
                    blsPrivateKey,
                };
                const encodedGeneratorKeys = lisk_codec_1.codec.encode(exports.plainGeneratorKeysSchema, plainGeneratorKeyData);
                encryptedMessageObject = await lisk_cryptography_1.encrypt.encryptMessageWithPassword(encodedGeneratorKeys, password);
            }
            keys.push({
                address: lisk_cryptography_1.address.getLisk32AddressFromAddress(address),
                keyPath: accountKeyPath,
                publicKey: accountPublicKey.toString('hex'),
                privateKey: accountPrivateKey.toString('hex'),
                plain: {
                    generatorKeyPath,
                    generatorKey: generatorPublicKey.toString('hex'),
                    generatorPrivateKey: generatorPrivateKey.toString('hex'),
                    blsKeyPath,
                    blsKey: blsPublicKey.toString('hex'),
                    blsProofOfPossession: lisk_cryptography_1.bls.popProve(blsPrivateKey).toString('hex'),
                    blsPrivateKey: blsPrivateKey.toString('hex'),
                },
                encrypted: encryptedMessageObject,
            });
        }
        if (output) {
            fs.writeJSONSync(output, { keys }, { spaces: ' ' });
        }
        else {
            this.log(JSON.stringify({ keys }, undefined, '  '));
        }
    }
}
CreateCommand.description = 'Return keys corresponding to the given passphrase.';
CreateCommand.examples = [
    'keys:create',
    'keys:create --passphrase your-passphrase',
    'keys:create --passphrase your-passphrase --no-encrypt',
    'keys:create --passphrase your-passphrase --password your-password',
    'keys:create --passphrase your-passphrase --password your-password --count 2',
    'keys:create --passphrase your-passphrase --no-encrypt --count 2 --offset 1',
    'keys:create --passphrase your-passphrase --no-encrypt --count 2 --offset 1 --chainid 1',
    'keys:create --passphrase your-passphrase --password your-password --count 2 --offset 1 --chainid 1 --output /mypath/keys.json',
];
CreateCommand.flags = {
    output: flags_1.flagsWithParser.output,
    passphrase: flags_1.flagsWithParser.passphrase,
    'no-encrypt': core_1.Flags.boolean({
        char: 'n',
        description: 'No encrypted message object to be created',
        default: false,
    }),
    password: flags_1.flagsWithParser.password,
    count: core_1.Flags.integer({
        char: 'c',
        description: 'Number of keys to create',
        default: 1,
    }),
    offset: core_1.Flags.integer({
        char: 'f',
        description: 'Offset for the key derivation path',
        default: 0,
    }),
    chainid: core_1.Flags.integer({
        char: 'i',
        description: 'Chain id',
        default: 0,
    }),
};
exports.CreateCommand = CreateCommand;
//# sourceMappingURL=create.js.map