"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseStateRecoveryCommand = void 0;
const lisk_utils_1 = require("@liskhq/lisk-utils");
const lisk_db_1 = require("@liskhq/lisk-db");
const lisk_cryptography_1 = require("@liskhq/lisk-cryptography");
const lisk_validator_1 = require("@liskhq/lisk-validator");
const base_interoperability_command_1 = require("./base_interoperability_command");
const constants_1 = require("./constants");
const schemas_1 = require("./schemas");
const state_machine_1 = require("../../state_machine");
const terminated_state_1 = require("./stores/terminated_state");
const base_store_1 = require("../base_store");
class BaseStateRecoveryCommand extends base_interoperability_command_1.BaseInteroperabilityCommand {
    constructor() {
        super(...arguments);
        this.schema = schemas_1.stateRecoveryParamsSchema;
    }
    async verify(context) {
        const { params: { chainID, storeEntries, siblingHashes, module }, } = context;
        try {
            lisk_validator_1.validator.validate(this.schema, context.params);
        }
        catch (err) {
            return {
                status: state_machine_1.VerifyStatus.FAIL,
                error: err,
            };
        }
        const terminatedStateSubstore = this.stores.get(terminated_state_1.TerminatedStateStore);
        const terminatedStateAccountExists = await terminatedStateSubstore.has(context, chainID);
        if (!terminatedStateAccountExists) {
            return {
                status: state_machine_1.VerifyStatus.FAIL,
                error: new Error('The terminated state does not exist.'),
            };
        }
        const terminatedStateAccount = await terminatedStateSubstore.get(context, chainID);
        if (!terminatedStateAccount.initialized) {
            return {
                status: state_machine_1.VerifyStatus.FAIL,
                error: new Error('The terminated state is not initialized.'),
            };
        }
        const moduleMethod = this.interoperableCCMethods.get(module);
        if (!moduleMethod) {
            return {
                status: state_machine_1.VerifyStatus.FAIL,
                error: new Error('Module is not registered on the chain.'),
            };
        }
        if (!moduleMethod.recover) {
            return {
                status: state_machine_1.VerifyStatus.FAIL,
                error: new Error('Module is not recoverable.'),
            };
        }
        const { stateRoot } = terminatedStateAccount;
        const queryKeys = [];
        const storeQueries = [];
        const storePrefix = (0, base_store_1.computeStorePrefix)(module);
        for (const entry of storeEntries) {
            if (entry.storeValue.equals(constants_1.EMPTY_BYTES)) {
                return {
                    status: state_machine_1.VerifyStatus.FAIL,
                    error: new Error('Recovered store value cannot be empty.'),
                };
            }
            const queryKey = Buffer.concat([
                storePrefix,
                entry.substorePrefix,
                lisk_cryptography_1.utils.hash(entry.storeKey),
            ]);
            queryKeys.push(queryKey);
            storeQueries.push({
                key: queryKey,
                value: lisk_cryptography_1.utils.hash(entry.storeValue),
                bitmap: entry.bitmap,
            });
        }
        if (!lisk_utils_1.objects.bufferArrayUniqueItems(queryKeys)) {
            return {
                status: state_machine_1.VerifyStatus.FAIL,
                error: new Error('Recovered store keys are not pairwise distinct.'),
            };
        }
        const proofOfInclusionStores = { siblingHashes, queries: storeQueries };
        const sparseMerkleTree = new lisk_db_1.SparseMerkleTree();
        const verified = await sparseMerkleTree.verify(stateRoot, queryKeys, proofOfInclusionStores);
        if (!verified) {
            return {
                status: state_machine_1.VerifyStatus.FAIL,
                error: new Error('State recovery proof of inclusion is not valid.'),
            };
        }
        return {
            status: state_machine_1.VerifyStatus.OK,
        };
    }
    async execute(context) {
        const { params: { chainID, storeEntries, module, siblingHashes }, } = context;
        const storeQueries = [];
        const moduleMethod = this.interoperableCCMethods.get(module);
        const storePrefix = (0, base_store_1.computeStorePrefix)(module);
        for (const entry of storeEntries) {
            try {
                await moduleMethod.recover({
                    ...context,
                    module,
                    terminatedChainID: chainID,
                    substorePrefix: entry.substorePrefix,
                    storeKey: entry.storeKey,
                    storeValue: entry.storeValue,
                });
            }
            catch (err) {
                throw new Error(`Recovery failed for module: ${module}`);
            }
            storeQueries.push({
                key: Buffer.concat([storePrefix, entry.substorePrefix, lisk_cryptography_1.utils.hash(entry.storeKey)]),
                value: constants_1.EMPTY_HASH,
                bitmap: entry.bitmap,
            });
        }
        const sparseMerkleTree = new lisk_db_1.SparseMerkleTree();
        const root = await sparseMerkleTree.calculateRoot({
            queries: storeQueries,
            siblingHashes,
        });
        const terminatedStateSubstore = this.stores.get(terminated_state_1.TerminatedStateStore);
        const terminatedStateAccount = await terminatedStateSubstore.get(context, chainID);
        await terminatedStateSubstore.set(context, chainID, {
            ...terminatedStateAccount,
            stateRoot: root,
        });
    }
}
exports.BaseStateRecoveryCommand = BaseStateRecoveryCommand;
//# sourceMappingURL=base_state_recovery.js.map