"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubmitMainchainCrossChainUpdateCommand = void 0;
const lisk_codec_1 = require("@liskhq/lisk-codec");
const lisk_cryptography_1 = require("@liskhq/lisk-cryptography");
const lisk_validator_1 = require("@liskhq/lisk-validator");
const schema_1 = require("../../../../engine/consensus/certificate_generation/schema");
const state_machine_1 = require("../../../../state_machine");
const base_cross_chain_update_command_1 = require("../../base_cross_chain_update_command");
const constants_1 = require("../../constants");
const ccm_processed_1 = require("../../events/ccm_processed");
const schemas_1 = require("../../schemas");
const chain_account_1 = require("../../stores/chain_account");
const chain_validators_1 = require("../../stores/chain_validators");
const utils_1 = require("../../utils");
class SubmitMainchainCrossChainUpdateCommand extends base_cross_chain_update_command_1.BaseCrossChainUpdateCommand {
    async verify(context) {
        const { params } = context;
        lisk_validator_1.validator.validate(schemas_1.crossChainUpdateTransactionParams, context.params);
        if (params.certificate.length === 0 && (0, utils_1.isInboxUpdateEmpty)(params.inboxUpdate)) {
            throw new Error('A cross-chain update must contain a non-empty certificate and/or a non-empty inbox update.');
        }
        const isLive = await this.internalMethod.isLive(context, params.sendingChainID, context.header.timestamp);
        if (!isLive) {
            throw new Error('The sending chain is not live.');
        }
        const sendingChainAccount = await this.stores
            .get(chain_account_1.ChainAccountStore)
            .get(context, params.sendingChainID);
        if (sendingChainAccount.status === 0 && params.certificate.length === 0) {
            throw new Error('Cross-chain updates from chains with status CHAIN_STATUS_REGISTERED must contain a non-empty certificate.');
        }
        if (params.certificate.length > 0) {
            await this.internalMethod.verifyCertificate(context, params, context.header.timestamp);
        }
        if (sendingChainAccount.status === 0 &&
            !(0, utils_1.isInboxUpdateEmpty)(params.inboxUpdate)) {
            this._verifyLivenessConditionForRegisteredChains(context);
        }
        const sendingChainValidators = await this.stores
            .get(chain_validators_1.ChainValidatorsStore)
            .get(context, params.sendingChainID);
        if (!(0, utils_1.emptyActiveValidatorsUpdate)(params.activeValidatorsUpdate) ||
            params.certificateThreshold !== sendingChainValidators.certificateThreshold) {
            await this.internalMethod.verifyValidatorsUpdate(context, params);
        }
        if (!(0, utils_1.isInboxUpdateEmpty)(params.inboxUpdate)) {
            await this.internalMethod.verifyPartnerChainOutboxRoot(context, params);
        }
        return {
            status: state_machine_1.VerifyStatus.OK,
        };
    }
    async execute(context) {
        const [decodedCCMs, ok] = await this.executeCommon(context, true);
        if (!ok) {
            return;
        }
        const { params } = context;
        try {
            context.contextStore.set(constants_1.CONTEXT_STORE_KEY_CCM_PROCESSING, true);
            for (let i = 0; i < decodedCCMs.length; i += 1) {
                const ccm = decodedCCMs[i];
                const ccmBytes = params.inboxUpdate.crossChainMessages[i];
                const ccmID = lisk_cryptography_1.utils.hash(ccmBytes);
                const ccmContext = {
                    ...context,
                    ccm,
                    eventQueue: context.eventQueue.getChildQueue(ccmID),
                };
                if (ccm.receivingChainID.equals((0, utils_1.getMainchainID)(context.chainID))) {
                    await this.apply(ccmContext);
                }
                else {
                    await this._forward(ccmContext);
                }
                await this.internalMethod.appendToInboxTree(context, params.sendingChainID, ccmBytes);
            }
        }
        finally {
            context.contextStore.delete(constants_1.CONTEXT_STORE_KEY_CCM_PROCESSING);
        }
        await this.afterExecuteCommon(context);
    }
    async _forward(context) {
        const { ccm, logger } = context;
        const { ccmID, encodedCCM } = (0, utils_1.getEncodedCCMAndID)(ccm);
        const valid = await this.verifyCCM(context, ccmID);
        if (!valid) {
            return;
        }
        let receivingChainAccount;
        try {
            receivingChainAccount = await this.stores
                .get(chain_account_1.ChainAccountStore)
                .get(context, ccm.receivingChainID);
            if (receivingChainAccount.status === 0) {
                await this.bounce(context, encodedCCM.length, 1, 1);
                return;
            }
        }
        catch (error) {
            if (error instanceof state_machine_1.NotFoundError) {
                await this.bounce(context, encodedCCM.length, 1, 1);
                return;
            }
            throw error;
        }
        const isLive = await this.internalMethod.isLive(context, ccm.receivingChainID, context.header.timestamp);
        if (!isLive) {
            await this.internalMethod.terminateChainInternal(context, ccm.receivingChainID);
            this.events.get(ccm_processed_1.CcmProcessedEvent).log(context, ccm.sendingChainID, ccm.receivingChainID, {
                code: 1,
                result: 3,
                ccm,
            });
            await this.internalMethod.sendInternal(context, constants_1.EMPTY_FEE_ADDRESS, constants_1.MODULE_NAME_INTEROPERABILITY, constants_1.CROSS_CHAIN_COMMAND_SIDECHAIN_TERMINATED, ccm.sendingChainID, BigInt(0), 0, lisk_codec_1.codec.encode(schemas_1.sidechainTerminatedCCMParamsSchema, {
                chainID: ccm.receivingChainID,
                stateRoot: receivingChainAccount.lastCertificate.stateRoot,
            }));
            return;
        }
        const stateSnapshotID = context.stateStore.createSnapshot();
        const eventSnapshotID = context.eventQueue.createSnapshot();
        try {
            for (const [module, method] of this.interoperableCCMethods.entries()) {
                if (method.beforeCrossChainMessageForwarding) {
                    logger.debug({
                        moduleName: module,
                        commandName: ccm.crossChainCommand,
                        ccmID: ccmID.toString('hex'),
                    }, 'Execute beforeCrossChainCommandExecute');
                    await method.beforeCrossChainMessageForwarding(context);
                }
            }
        }
        catch (error) {
            context.eventQueue.restoreSnapshot(eventSnapshotID);
            context.stateStore.restoreSnapshot(stateSnapshotID);
            logger.info({ err: error, moduleName: ccm.module, commandName: ccm.crossChainCommand }, 'Fail to execute beforeCrossChainCommandExecute.');
            await this.internalMethod.terminateChainInternal(context, ccm.sendingChainID);
            this.events.get(ccm_processed_1.CcmProcessedEvent).log(context, ccm.sendingChainID, ccm.receivingChainID, {
                code: 12,
                result: 3,
                ccm,
            });
            return;
        }
        await this.internalMethod.addToOutbox(context, ccm.receivingChainID, ccm);
        this.events.get(ccm_processed_1.CcmProcessedEvent).log(context, ccm.sendingChainID, ccm.receivingChainID, {
            code: 0,
            result: 1,
            ccm,
        });
    }
    _verifyLivenessConditionForRegisteredChains(context) {
        const certificate = lisk_codec_1.codec.decode(schema_1.certificateSchema, context.params.certificate);
        if (context.header.timestamp - certificate.timestamp > constants_1.LIVENESS_LIMIT / 2) {
            throw new Error(`The first CCU with a non-empty inbox update cannot contain a certificate older than ${constants_1.LIVENESS_LIMIT / 2} seconds.`);
        }
    }
}
exports.SubmitMainchainCrossChainUpdateCommand = SubmitMainchainCrossChainUpdateCommand;
//# sourceMappingURL=submit_mainchain_cross_chain_update.js.map